@@ -1,6 +1,10 @@
 # -*- coding: utf-8 -*-
 
-from django.conf.urls.defaults import *
+try:
+    from django.conf.urls import *
+except ImportError: # django < 1.4
+    from django.conf.urls.defaults import *
+
 from .views import RedisStatsView
 
 urlpatterns = patterns('',
